@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +16,8 @@ gi.require_version('WebKit2', '4.0')
 gi.require_version('Soup', '2.4')
 gi.require_version('Secret', '1')
 gi.require_version('GtkSpell', '3.0')
-from gi.repository import Gtk, Gio, GLib, Gdk, WebKit2
+gi.require_version("Handy", "1")
+from gi.repository import Gtk, Gio, GLib, Gdk, WebKit2, Handy
 
 from threading import current_thread
 from gettext import gettext as _
@@ -46,6 +47,7 @@ from eolie.download_manager import DownloadManager
 from eolie.menu_pages import PagesMenu
 from eolie.helper_task import TaskHelper
 from eolie.define import EOLIE_DATA_PATH, TimeSpan, TimeSpanValues, LoadingType
+from eolie.define import StartPage
 from eolie.utils import is_unity, wanted_loading_type
 from eolie.logger import Logger
 from eolie.webview_state import WebViewState
@@ -58,15 +60,17 @@ class Application(Gtk.Application, NightApplication):
 
     __FAVICONS_PATH = "/tmp/eolie_%s" % getuser()
 
-    def __init__(self, version, data_dir):
+    def __init__(self, version, data_dir, app_id):
         """
             Create application
             @param version as str
             @param data_dir as str
+            @param app_id as str
         """
         self.__version = version
         self.__state_cache = []
         self.__data_dir = data_dir
+        self.__app_id = app_id
         self.__content_blockers = []
         signal(SIGINT, lambda a, b: self.quit())
         signal(SIGTERM, lambda a, b: self.quit())
@@ -79,7 +83,7 @@ class Application(Gtk.Application, NightApplication):
             exit("You need WebKit2GTK >= 2.20")
         Gtk.Application.__init__(
             self,
-            application_id="org.gnome.Eolie",
+            application_id=app_id,
             flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
         self.set_property("register-session", True)
         # Fix proxy for python
@@ -89,16 +93,11 @@ class Application(Gtk.Application, NightApplication):
             from os import environ
             environ["all_proxy"] = proxy
             environ["ALL_PROXY"] = proxy
-        # Ideally, we will be able to delete this once Flatpak has a solution
-        # for SSL certificate management inside of applications.
+        # Flatpak Hacks
         if GLib.file_test("/app", GLib.FileTest.EXISTS):
-            paths = ["/etc/ssl/certs/ca-certificates.crt",
-                     "/etc/pki/tls/cert.pem",
-                     "/etc/ssl/cert.pem"]
-            for path in paths:
-                if GLib.file_test(path, GLib.FileTest.EXISTS):
-                    GLib.setenv("SSL_CERT_FILE", path, True)
-                    break
+            # Set /tmp for GLib, /tmp not accessible in flatpak
+            tmp = GLib.environ_getenv(GLib.get_environ(), "XDG_RUNTIME_DIR")
+            GLib.setenv("TMPDIR", "%s/app/org.gnome.Eolie" % tmp, True)
         self.sync_worker = None  # Not initialised
         self.show_tls = False
         self.cursors = {}
@@ -128,14 +127,6 @@ class Application(Gtk.Application, NightApplication):
         else:
             gc.disable()
 
-    def do_startup(self):
-        """
-            Init application
-        """
-        Gtk.Application.do_startup(self)
-        if not self.get_windows():
-            self.__init()
-
     def get_new_window(self):
         """
             Return a new window
@@ -208,9 +199,9 @@ class Application(Gtk.Application, NightApplication):
                 s = str(x)
                 print(type(x), "\n  ", s)
         if vacuum:
-            task_helper = TaskHelper()
-            task_helper.run(self.__vacuum,
-                            callback=(lambda x: Gio.Application.quit(self),))
+            self.task_helper.run(
+                        self.__vacuum,
+                        callback=(lambda x: Gio.Application.quit(self),))
         else:
             Gio.Application.quit(self)
 
@@ -242,15 +233,17 @@ class Application(Gtk.Application, NightApplication):
             Get start page
             @return uri as str
         """
-        value = self.settings.get_value("start-page").get_string()
-        if value in ["popular_hist", "popular_book"]:
+        value = self.settings.get_enum("start-page")
+        if value in [StartPage.POPULARITY_HISTORY,
+                     StartPage.POPULARITY_BOOKMARKS]:
             start_page = "populars://"
-        elif value == "blank":
+        elif value == StartPage.BLANK:
             start_page = "about:blank"
-        elif value == "search":
+        elif value == StartPage.SEARCH:
             start_page = self.search.uri
         else:
-            start_page = value
+            start_page = self.settings.get_value(
+                "start-page-custom").get_string()
         return start_page
 
     @property
@@ -292,6 +285,7 @@ class Application(Gtk.Application, NightApplication):
         """
             Init main application
         """
+        Handy.init()
         self.settings = Settings.new()
         NightApplication.__init__(self)
 
@@ -310,6 +304,15 @@ class Application(Gtk.Application, NightApplication):
         styleContext = Gtk.StyleContext()
         styleContext.add_provider_for_screen(screen, cssProvider,
                                              Gtk.STYLE_PROVIDER_PRIORITY_USER)
+        # Devel version, load devel theme
+        if self.__app_id is None:
+            css = ".progressbar progress { background-color: red; }"
+            cssProvider = Gtk.CssProvider()
+            cssProvider.load_from_data(css.encode("utf-8"))
+            screen = Gdk.Screen.get_default()
+            styleContext = Gtk.StyleContext()
+            styleContext.add_provider_for_screen(
+                screen, cssProvider, Gtk.STYLE_PROVIDER_PRIORITY_USER + 1)
         self.history = DatabaseHistory()
         self.bookmarks = DatabaseBookmarks()
         self.websettings = DatabaseSettings()
@@ -331,6 +334,7 @@ class Application(Gtk.Application, NightApplication):
         settings = WebKit2.Settings.new()
         self.search = Search(settings.get_user_agent())
 
+        self.task_helper = TaskHelper()
         self.download_manager = DownloadManager()
         self.pages_menu = PagesMenu()
 
@@ -539,6 +543,7 @@ class Application(Gtk.Application, NightApplication):
 
         # Only restore state on first run
         if not self.windows:
+            self.__init()
             self.__restore_state()
 
         # Setup at least one window
